@@ -23,6 +23,7 @@
 #include <config.h>
 #endif
 
+#include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -34,7 +35,7 @@
 #include "gcm-calibrate-argyll.h"
 #include "gcm-cie-widget.h"
 #include "gcm-client.h"
-#include "gcm-colorimeter.h"
+#include "gcm-sensor-client.h"
 #include "gcm-device-xrandr.h"
 #include "gcm-device-virtual.h"
 #include "gcm-exif.h"
@@ -55,13 +56,15 @@ struct _CcColorPanelPrivate {
 	GcmDevice		*current_device;
 	GcmProfileStore		*profile_store;
 	GcmClient		*gcm_client;
-	GcmColorimeter		*colorimeter;
+	GcmSensorClient		*sensor_client;
 	gboolean		 setting_up_device;
 	GtkWidget		*main_window;
 	GtkWidget		*info_bar_loading;
 	GtkWidget		*info_bar_vcgt;
 	GtkWidget		*info_bar_profiles;
 	GSettings		*settings;
+	guint			 save_and_apply_id;
+	guint			 apply_all_devices_id;
 };
 
 G_DEFINE_DYNAMIC_TYPE (CcColorPanel, cc_color_panel, CC_TYPE_PANEL)
@@ -1283,21 +1286,72 @@ cc_color_panel_delete_cb (GtkWidget *widget, CcColorPanel *panel)
 	}
 }
 
+
+/**
+ * cc_color_panel_save_and_apply_current_device_cb:
+ **/
+static gboolean
+cc_color_panel_save_and_apply_current_device_cb (CcColorPanel *panel)
+{
+	gfloat localgamma;
+	gfloat brightness;
+	gfloat contrast;
+	GtkWidget *widget;
+	gboolean ret;
+	GError *error = NULL;
+
+	/* get values */
+	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_gamma"));
+	localgamma = gtk_range_get_value (GTK_RANGE (widget));
+	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_brightness"));
+	brightness = gtk_range_get_value (GTK_RANGE (widget));
+	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_contrast"));
+	contrast = gtk_range_get_value (GTK_RANGE (widget));
+
+	gcm_device_set_gamma (panel->priv->current_device, localgamma);
+	gcm_device_set_brightness (panel->priv->current_device, brightness * 100.0f);
+	gcm_device_set_contrast (panel->priv->current_device, contrast * 100.0f);
+
+	/* save new profile */
+	ret = gcm_device_save (panel->priv->current_device, &error);
+	if (!ret) {
+		egg_warning ("failed to save config: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* actually set the new profile */
+	ret = gcm_device_apply (panel->priv->current_device, &error);
+	if (!ret) {
+		egg_warning ("failed to apply profile: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	panel->priv->save_and_apply_id = 0;
+	return FALSE;
+}
+
 /**
  * cc_color_panel_reset_cb:
  **/
 static void
 cc_color_panel_reset_cb (GtkWidget *widget, CcColorPanel *panel)
 {
-	panel->priv->setting_up_device = TRUE;
 	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_gamma"));
 	gtk_range_set_value (GTK_RANGE (widget), 1.0f);
 	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_brightness"));
 	gtk_range_set_value (GTK_RANGE (widget), 0.0f);
-	panel->priv->setting_up_device = FALSE;
-	/* we only want one save, not three */
 	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_contrast"));
 	gtk_range_set_value (GTK_RANGE (widget), 1.0f);
+
+	/* if we've already queued a save and apply, ignore this */
+	if (panel->priv->save_and_apply_id != 0) {
+		egg_debug ("ignoring extra save and apply, as one is already pending");
+		return;
+	}
+	panel->priv->save_and_apply_id =
+		g_timeout_add (50, (GSourceFunc) cc_color_panel_save_and_apply_current_device_cb, panel);
 }
 
 /**
@@ -1371,8 +1425,6 @@ cc_color_panel_set_calibrate_button_sensitivity (CcColorPanel *panel)
 	GtkWidget *widget;
 	const gchar *tooltip;
 	GcmDeviceKind kind;
-	gboolean connected;
-	gboolean xrandr_fallback;
 	gboolean has_vte = TRUE;
 
 	/* TRANSLATORS: this is when the button is sensitive */
@@ -1401,23 +1453,23 @@ cc_color_panel_set_calibrate_button_sensitivity (CcColorPanel *panel)
 	if (kind == GCM_DEVICE_KIND_DISPLAY) {
 
 		/* are we disconnected */
-		connected = gcm_device_get_connected (panel->priv->current_device);
-		if (!connected) {
+		ret = gcm_device_get_connected (panel->priv->current_device);
+		if (!ret) {
 			/* TRANSLATORS: this is when the button is insensitive */
 			tooltip = _("Cannot create profile: The display device is not connected");
 			goto out;
 		}
 
 		/* are we not XRandR 1.3 compat */
-		xrandr_fallback = gcm_device_xrandr_get_fallback (GCM_DEVICE_XRANDR (panel->priv->current_device));
-		if (xrandr_fallback) {
+		ret = gcm_device_xrandr_get_xrandr13 (GCM_DEVICE_XRANDR (panel->priv->current_device));
+		if (!ret) {
 			/* TRANSLATORS: this is when the button is insensitive */
 			tooltip = _("Cannot create profile: The display driver does not support XRandR 1.3");
 			goto out;
 		}
 
 		/* find whether we have hardware installed */
-		ret = gcm_colorimeter_get_present (panel->priv->colorimeter);
+		ret = gcm_sensor_client_get_present (panel->priv->sensor_client);
 		if (!ret) {
 			/* TRANSLATORS: this is when the button is insensitive */
 			tooltip = _("Cannot create profile: The measuring instrument is not plugged in");
@@ -1432,7 +1484,7 @@ cc_color_panel_set_calibrate_button_sensitivity (CcColorPanel *panel)
 	} else if (kind == GCM_DEVICE_KIND_PRINTER) {
 
 		/* find whether we have hardware installed */
-		ret = gcm_colorimeter_get_present (panel->priv->colorimeter);
+		ret = gcm_sensor_client_get_present (panel->priv->sensor_client);
 		if (!ret) {
 			/* TRANSLATORS: this is when the button is insensitive */
 			tooltip = _("Cannot create profile: The measuring instrument is not plugged in");
@@ -1440,7 +1492,7 @@ cc_color_panel_set_calibrate_button_sensitivity (CcColorPanel *panel)
 		}
 
 		/* find whether we have hardware installed */
-		ret = gcm_colorimeter_supports_printer (panel->priv->colorimeter);
+		ret = gcm_sensor_supports_printer (gcm_sensor_client_get_sensor (panel->priv->sensor_client));
 		if (!ret) {
 			/* TRANSLATORS: this is when the button is insensitive */
 			tooltip = _("Cannot create profile: The measuring instrument does not support printer profiling");
@@ -1522,19 +1574,20 @@ cc_color_panel_devices_treeview_clicked_cb (GtkTreeSelection *selection, CcColor
 
 	/* show broken devices */
 	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "label_problems"));
+	gtk_widget_hide (widget);
 	if (kind == GCM_DEVICE_KIND_DISPLAY) {
-		ret = gcm_device_xrandr_get_fallback (GCM_DEVICE_XRANDR (panel->priv->current_device));
+		ret = gcm_device_get_connected (panel->priv->current_device);
 		if (ret) {
-			/* TRANSLATORS: Some shitty binary drivers do not support per-head gamma controls.
-			 * Whilst this does not matter if you only have one monitor attached, it means you
-			 * can't color correct additional monitors or projectors. */
-			gtk_label_set_label (GTK_LABEL (widget), _("Per-device settings not supported. Check your display driver."));
-			gtk_widget_show (widget);
-		} else {
-			gtk_widget_hide (widget);
+			ret = gcm_device_xrandr_get_xrandr13 (GCM_DEVICE_XRANDR (panel->priv->current_device));
+			if (!ret) {
+				/* TRANSLATORS: Some shitty binary drivers do not support per-head gamma controls.
+				* Whilst this does not matter if you only have one monitor attached, it means you
+				* can't color correct additional monitors or projectors. */
+				gtk_label_set_label (GTK_LABEL (widget),
+						     _("Per-device settings not supported. Check your display driver."));
+				gtk_widget_show (widget);
+			}
 		}
-	} else {
-		gtk_widget_hide (widget);
 	}
 
 	/* set device labels */
@@ -1910,59 +1963,34 @@ out:
 static void
 cc_color_panel_slider_changed_cb (GtkRange *range, CcColorPanel *panel)
 {
-	gfloat localgamma;
-	gfloat brightness;
-	gfloat contrast;
-	GtkWidget *widget;
-	gboolean ret;
-	GError *error = NULL;
-
 	/* we're just setting up the device, not moving the slider */
-	if (panel->priv->setting_up_device)
+	if (panel->priv->setting_up_device) {
+		egg_debug ("setting up device, so ignore");
 		return;
-
-	/* get values */
-	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_gamma"));
-	localgamma = gtk_range_get_value (GTK_RANGE (widget));
-	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_brightness"));
-	brightness = gtk_range_get_value (GTK_RANGE (widget));
-	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "hscale_contrast"));
-	contrast = gtk_range_get_value (GTK_RANGE (widget));
-
-	gcm_device_set_gamma (panel->priv->current_device, localgamma);
-	gcm_device_set_brightness (panel->priv->current_device, brightness * 100.0f);
-	gcm_device_set_contrast (panel->priv->current_device, contrast * 100.0f);
-
-	/* save new profile */
-	ret = gcm_device_save (panel->priv->current_device, &error);
-	if (!ret) {
-		egg_warning ("failed to save config: %s", error->message);
-		g_error_free (error);
-		goto out;
 	}
 
-	/* actually set the new profile */
-	ret = gcm_device_apply (panel->priv->current_device, &error);
-	if (!ret) {
-		egg_warning ("failed to apply profile: %s", error->message);
-		g_error_free (error);
-		goto out;
+	/* if we've already queued a save and apply, ignore this */
+	if (panel->priv->save_and_apply_id != 0) {
+		egg_debug ("ignoring extra save and apply, as one is already pending");
+		return;
 	}
-out:
-	return;
+
+	/* schedule a save and apply */
+	panel->priv->save_and_apply_id =
+		g_timeout_add (50, (GSourceFunc) cc_color_panel_save_and_apply_current_device_cb, panel);
 }
 
 /**
- * cc_color_panel_colorimeter_changed_cb:
+ * cc_color_panel_sensor_client_changed_cb:
  **/
 static void
-cc_color_panel_colorimeter_changed_cb (GcmColorimeter *colorimeter, CcColorPanel *panel)
+cc_color_panel_sensor_client_changed_cb (GcmSensorClient *sensor_client, CcColorPanel *panel)
 {
 	gboolean present;
 	const gchar *event_id;
 	const gchar *message;
 
-	present = gcm_colorimeter_get_present (colorimeter);
+	present = gcm_sensor_client_get_present (sensor_client);
 
 	if (present) {
 		/* TRANSLATORS: this is a sound description */
@@ -2200,7 +2228,7 @@ cc_color_panel_setup_space_combobox (CcColorPanel *panel, GtkWidget *widget, Gcm
 		colorspace_tmp = gcm_profile_get_colorspace (profile);
 		if (!has_vcgt &&
 		    colorspace == colorspace_tmp &&
-		    (colorspace == GCM_COLORSPACE_CMYK ||
+		    (colorspace != GCM_COLORSPACE_RGB ||
 		     has_colorspace_description)) {
 			cc_color_panel_combobox_add_profile (widget, profile, GCM_PREFS_ENTRY_TYPE_PROFILE, &iter);
 
@@ -2314,11 +2342,18 @@ cc_color_panel_startup_idle_cb (CcColorPanel *panel)
 	GError *error = NULL;
 	gchar *colorspace_rgb;
 	gchar *colorspace_cmyk;
+	gchar *colorspace_gray;
 	gint intent_display = -1;
 	gint intent_softproof = -1;
+	GcmProfileSearchFlags search_flags = GCM_PROFILE_STORE_SEARCH_ALL;
+
+	/* volume checking is optional */
+	ret = g_settings_get_boolean (panel->priv->settings, GCM_SETTINGS_USE_PROFILES_FROM_VOLUMES);
+	if (!ret)
+		search_flags &= ~GCM_PROFILE_STORE_SEARCH_VOLUMES;
 
 	/* search the disk for profiles */
-	gcm_profile_store_search (panel->priv->profile_store, GCM_PROFILE_STORE_SEARCH_ALL);
+	gcm_profile_store_search (panel->priv->profile_store, search_flags);
 	g_signal_connect (panel->priv->profile_store, "changed", G_CALLBACK(cc_color_panel_profile_store_changed_cb), panel);
 
 	/* setup RGB combobox */
@@ -2339,6 +2374,15 @@ cc_color_panel_startup_idle_cb (CcColorPanel *panel)
 	g_signal_connect (G_OBJECT (widget), "changed",
 			  G_CALLBACK (cc_color_panel_space_combo_changed_cb), panel);
 
+	/* setup gray combobox */
+	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "combobox_space_gray"));
+	colorspace_gray = g_settings_get_string (panel->priv->settings, GCM_SETTINGS_COLORSPACE_GRAY);
+	cc_color_panel_set_combo_simple_text (widget);
+	cc_color_panel_setup_space_combobox (panel, widget, GCM_COLORSPACE_GRAY, colorspace_gray);
+	g_object_set_data (G_OBJECT(widget), "GCM:GSettingsKey", (gpointer) GCM_SETTINGS_COLORSPACE_GRAY);
+	g_signal_connect (G_OBJECT (widget), "changed",
+			  G_CALLBACK (cc_color_panel_space_combo_changed_cb), panel);
+
 	/* setup rendering lists */
 	widget = GTK_WIDGET (gtk_builder_get_object (panel->priv->builder, "combobox_rendering_display"));
 	cc_color_panel_set_combo_simple_text (widget);
@@ -2381,10 +2425,10 @@ out:
 }
 
 /**
- * cc_color_panel_reset_devices_idle_cb:
+ * cc_color_panel_apply_all_devices_idle_cb:
  **/
 static gboolean
-cc_color_panel_reset_devices_idle_cb (CcColorPanel *panel)
+cc_color_panel_apply_all_devices_idle_cb (CcColorPanel *panel)
 {
 	GPtrArray *array = NULL;
 	GcmDevice *device;
@@ -2397,6 +2441,11 @@ cc_color_panel_reset_devices_idle_cb (CcColorPanel *panel)
 	for (i=0; i<array->len; i++) {
 		device = g_ptr_array_index (array, i);
 
+		/* we don't care */
+		ret = gcm_device_get_connected (device);
+		if (!ret)
+			continue;
+
 		/* set gamma for device */
 		ret = gcm_device_apply (device, &error);
 		if (!ret) {
@@ -2406,6 +2455,7 @@ cc_color_panel_reset_devices_idle_cb (CcColorPanel *panel)
 		}
 	}
 	g_ptr_array_unref (array);
+	panel->priv->apply_all_devices_id = 0;
 	return FALSE;
 }
 
@@ -2416,7 +2466,8 @@ static void
 cc_color_panel_checkbutton_changed_cb (GtkWidget *widget, CcColorPanel *panel)
 {
 	/* set the new setting */
-	g_idle_add ((GSourceFunc) cc_color_panel_reset_devices_idle_cb, panel);
+	panel->priv->apply_all_devices_id =
+		g_idle_add ((GSourceFunc) cc_color_panel_apply_all_devices_idle_cb, panel);
 }
 
 /**
@@ -2596,8 +2647,8 @@ cc_color_panel_finalize (GObject *object)
 
 	if (panel->priv->current_device != NULL)
 		g_object_unref (panel->priv->current_device);
-	if (panel->priv->colorimeter != NULL)
-		g_object_unref (panel->priv->colorimeter);
+	if (panel->priv->sensor_client != NULL)
+		g_object_unref (panel->priv->sensor_client);
 	if (panel->priv->settings != NULL)
 		g_object_unref (panel->priv->settings);
 	if (panel->priv->builder != NULL)
@@ -2606,6 +2657,10 @@ cc_color_panel_finalize (GObject *object)
 		g_object_unref (panel->priv->profile_store);
 	if (panel->priv->gcm_client != NULL)
 		g_object_unref (panel->priv->gcm_client);
+	if (panel->priv->save_and_apply_id != 0)
+		g_source_remove (panel->priv->save_and_apply_id);
+	if (panel->priv->apply_all_devices_id != 0)
+		g_source_remove (panel->priv->apply_all_devices_id);
 
 	G_OBJECT_CLASS (cc_color_panel_parent_class)->finalize (object);
 }
@@ -2802,8 +2857,8 @@ cc_color_panel_init (CcColorPanel *panel)
 			  G_CALLBACK (cc_color_panel_client_notify_loading_cb), panel);
 
 	/* use the color device */
-	panel->priv->colorimeter = gcm_colorimeter_new ();
-	g_signal_connect (panel->priv->colorimeter, "changed", G_CALLBACK (cc_color_panel_colorimeter_changed_cb), panel);
+	panel->priv->sensor_client = gcm_sensor_client_new ();
+	g_signal_connect (panel->priv->sensor_client, "changed", G_CALLBACK (cc_color_panel_sensor_client_changed_cb), panel);
 
 	/* use infobar */
 	panel->priv->info_bar_loading = gtk_info_bar_new ();
