@@ -39,6 +39,7 @@ int
 main (int argc, char **argv)
 {
 	gboolean ret;
+	gboolean login = FALSE;
 	guint retval = 0;
 	GError *error = NULL;
 	GOptionContext *context;
@@ -47,6 +48,13 @@ main (int argc, char **argv)
 	GcmClient *client = NULL;
 	GcmDevice *device;
 
+	const GOptionEntry options[] = {
+		{ "login", 'l', 0, G_OPTION_ARG_NONE, &login,
+		  /* TRANSLATORS: we use this mode at login as we're sure there are no previous settings to clear */
+		  _("Do not attempt to clear previously applied settings"), NULL },
+		{ NULL}
+	};
+
 	setlocale (LC_ALL, "");
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -56,14 +64,27 @@ main (int argc, char **argv)
 	gtk_init (&argc, &argv);
 
 	context = g_option_context_new ("gnome-color-manager apply program");
+	g_option_context_add_main_entries (context, options, NULL);
 	g_option_context_add_group (context, egg_debug_get_option_group ());
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_parse (context, &argc, &argv, NULL);
 	g_option_context_free (context);
 
+	/*
+	 * FIXME: THIS IS A HACK.
+	 * When gnome-settings-daemon ports from GConf to GSettings we can do this
+	 * properly and install a schema file. Until then, eugh.
+	 */
+	ret = g_spawn_command_line_async ("gconftool-2 --set --type=bool /apps/gnome_settings_daemon/plugins/color/active true", &error);
+	if (!ret) {
+		egg_warning ("failed to install GConf key: %s", error->message);
+		/* non-fatal */
+		g_clear_error (&error);
+	}
+
 	/* get devices */
 	client = gcm_client_new ();
-	ret = gcm_client_add_connected (client, &error);
+	ret = gcm_client_coldplug (client, GCM_CLIENT_COLDPLUG_XRANDR, &error);
 	if (!ret) {
 		egg_warning ("failed to get devices: %s", error->message);
 		g_error_free (error);
@@ -75,6 +96,9 @@ main (int argc, char **argv)
 	for (i=0; i<array->len; i++) {
 		device = g_ptr_array_index (array, i);
 
+		/* optimize for login to save a few hundred ms */
+		gcm_device_xrandr_set_remove_atom (GCM_DEVICE_XRANDR (device), !login);
+
 		/* set gamma for device */
 		egg_debug ("setting profiles on device: %s", gcm_device_get_id (device));
 		ret = gcm_device_apply (device, &error);
