@@ -694,51 +694,11 @@ gcm_calibrate_manual_finalize (GObject *object)
  *
  * Return value: a new GcmCalibrateManual object.
  **/
-GcmCalibrateManual *
+GcmCalibrate *
 gcm_calibrate_manual_new (void)
 {
 	GcmCalibrateManual *calibrate;
 	calibrate = g_object_new (GCM_TYPE_CALIBRATE_MANUAL, NULL);
-	return GCM_CALIBRATE_MANUAL (calibrate);
+	return GCM_CALIBRATE (calibrate);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-gcm_calibrate_manual_test (EggTest *test)
-{
-	GcmCalibrateManual *calibrate;
-	gboolean ret;
-	GError *error = NULL;
-
-	if (!egg_test_start (test, "GcmCalibrateManual"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get a calibrate_manual object");
-	calibrate = gcm_calibrate_manual_new ();
-	egg_test_assert (test, calibrate != NULL);
-
-	/* set to avoid a critical warning */
-	g_object_set (calibrate,
-		      "output-name", "lvds1",
-		      NULL);
-
-	/************************************************************/
-	egg_test_title (test, "calibrate display manually");
-	ret = gcm_calibrate_display (GCM_CALIBRATE(calibrate), NULL, &error);
-	if (ret)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "error: %s", error->message);
-
-	g_object_unref (calibrate);
-
-	egg_test_end (test);
-}
-#endif
-
