@@ -59,6 +59,9 @@ struct _GcmCalibrateClass
 	gboolean	 (*calibrate_printer)		(GcmCalibrate	*calibrate,
 							 GtkWindow	*window,
 							 GError		**error);
+	gboolean	 (*calibrate_spotread)		(GcmCalibrate	*calibrate,
+							 GtkWindow	*window,
+							 GError		**error);
 	/* padding for future expansion */
 	void (*_gcm_reserved1) (void);
 	void (*_gcm_reserved2) (void);
@@ -67,8 +70,7 @@ struct _GcmCalibrateClass
 	void (*_gcm_reserved5) (void);
 };
 
-typedef enum
-{
+typedef enum {
 	GCM_CALIBRATE_ERROR_USER_ABORT,
 	GCM_CALIBRATE_ERROR_NO_SUPPORT,
 	GCM_CALIBRATE_ERROR_NO_DATA,
@@ -124,6 +126,9 @@ gboolean	 gcm_calibrate_device			(GcmCalibrate	*calibrate,
 gboolean	 gcm_calibrate_printer			(GcmCalibrate	*calibrate,
 							 GtkWindow	*window,
 							 GError		**error);
+gboolean	 gcm_calibrate_spotread			(GcmCalibrate	*calibrate,
+							 GtkWindow	*window,
+							 GError		**error);
 gboolean	 gcm_calibrate_set_from_device		(GcmCalibrate	*calibrate,
 							 GcmDevice	*device,
 							 GError		**error);
