@@ -23,9 +23,9 @@
 #define __GCM_DEVICE_XRANDR_H
 
 #include <glib-object.h>
-#include <libgnomeui/gnome-rr.h>
 
 #include "gcm-device.h"
+#include "gcm-x11-screen.h"
 
 G_BEGIN_DECLS
 
@@ -51,13 +51,15 @@ struct _GcmDeviceXrandrClass
 GType		 gcm_device_xrandr_get_type		(void);
 GcmDevice	*gcm_device_xrandr_new			(void);
 gboolean	 gcm_device_xrandr_set_from_output	(GcmDevice		*device,
-							 GnomeRROutput		*output,
+							 GcmX11Output		*output,
 							 GError			**error);
 void		 gcm_device_xrandr_set_remove_atom	(GcmDeviceXrandr	*device_xrandr,
 							 gboolean		 remove_atom);
 const gchar	*gcm_device_xrandr_get_native_device	(GcmDeviceXrandr	*device_xrandr);
 const gchar	*gcm_device_xrandr_get_eisa_id		(GcmDeviceXrandr	*device_xrandr);
-gboolean	 gcm_device_xrandr_get_fallback		(GcmDeviceXrandr	*device_xrandr);
+const gchar	*gcm_device_xrandr_get_edid_md5		(GcmDeviceXrandr	*device_xrandr);
+gboolean	 gcm_device_xrandr_get_xrandr13		(GcmDeviceXrandr	*device_xrandr);
+gboolean	 gcm_device_xrandr_is_primary		(GcmDeviceXrandr	*device_xrandr);
 
 G_END_DECLS
 
