@@ -41,10 +41,9 @@ struct GcmGammaWidgetPrivate
 	gdouble			 color_blue;
 	guint			 chart_width;
 	guint			 chart_height;
-	cairo_t			*cr;
 };
 
-static gboolean gcm_gamma_widget_expose (GtkWidget *gamma, GdkEventExpose *event);
+static gboolean gcm_gamma_widget_draw (GtkWidget *gamma, cairo_t *cr);
 static void	gcm_gamma_widget_finalize (GObject *object);
 
 enum
@@ -130,7 +129,7 @@ gcm_gamma_widget_class_init (GcmGammaWidgetClass *class)
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
 
-	widget_class->expose_event = gcm_gamma_widget_expose;
+	widget_class->draw = gcm_gamma_widget_draw;
 	object_class->get_property = dkp_gamma_get_property;
 	object_class->set_property = dkp_gamma_set_property;
 	object_class->finalize = gcm_gamma_widget_finalize;
@@ -279,20 +278,22 @@ gcm_gamma_widget_draw_bounding_box (cairo_t *cr, gint x, gint y, gint width, gin
 
 /**
  * gcm_gamma_widget_draw:
+ *
+ * Just repaint the entire gamma widget on expose.
  **/
-static void
+static gboolean
 gcm_gamma_widget_draw (GtkWidget *gamma_widget, cairo_t *cr)
 {
 	GtkAllocation allocation;
 
 	GcmGammaWidget *gama = (GcmGammaWidget*) gamma_widget;
-	g_return_if_fail (gama != NULL);
-	g_return_if_fail (GCM_IS_GAMMA_WIDGET (gama));
+	g_return_val_if_fail (gama != NULL, FALSE);
+	g_return_val_if_fail (GCM_IS_GAMMA_WIDGET (gama), FALSE);
 
 	/* make size adjustment */
 	gtk_widget_get_allocation (gamma_widget, &allocation);
 	if (allocation.height <= 5 || allocation.width <= 5)
-		return;
+		return FALSE;
 
 	/* save */
 	gama->priv->chart_height = ((guint) (allocation.height / 2) * 2) - 1;
@@ -302,29 +303,6 @@ gcm_gamma_widget_draw (GtkWidget *gamma_widget, cairo_t *cr)
 	gcm_gamma_widget_draw_bounding_box (cr, 0, 0, gama->priv->chart_width, gama->priv->chart_height);
 	gcm_gamma_widget_draw_lines (gama, cr);
 	gcm_gamma_widget_draw_box (gama, cr);
-}
-
-/**
- * gcm_gamma_widget_expose:
- *
- * Just repaint the entire gamma widget on expose.
- **/
-static gboolean
-gcm_gamma_widget_expose (GtkWidget *gamma_widget, GdkEventExpose *event)
-{
-	cairo_t *cr;
-
-	/* get a cairo_t */
-	cr = gdk_cairo_create (gtk_widget_get_window (gamma_widget));
-	cairo_rectangle (cr,
-			 event->area.x, event->area.y,
-			 event->area.width, event->area.height);
-	cairo_clip (cr);
-	((GcmGammaWidget *)gamma_widget)->priv->cr = cr;
-
-	gcm_gamma_widget_draw (gamma_widget, cr);
-
-	cairo_destroy (cr);
 	return FALSE;
 }
 
@@ -338,67 +316,3 @@ gcm_gamma_widget_new (void)
 	return g_object_new (GCM_TYPE_GAMMA_WIDGET, NULL);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-gcm_gamma_widget_test (EggTest *test)
-{
-	GtkWidget *widget;
-	GtkWidget *image;
-	GtkWidget *dialog;
-	GtkWidget *vbox;
-	gboolean ret;
-	GError *error = NULL;
-	gint response;
-	gchar *filename_image;
-
-	if (!egg_test_start (test, "GcmGammaWidget"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get a gamma widget object");
-	widget = gcm_gamma_widget_new ();
-	egg_test_assert (test, widget != NULL);
-
-	g_object_set (widget,
-		      "color-light", 0.5f,
-		      "color-dark", 0.0f,
-		      "color-red", 0.25f,
-		      "color-green", 0.25f,
-		      "color-blue", 0.25f,
-		      NULL);
-
-	/************************************************************/
-	egg_test_title (test, "get filename of image file");
-	filename_image = egg_test_get_data_file ("gamma-widget.png");
-	egg_test_assert (test, (filename_image != NULL));
-
-	/* show in a dialog as an example */
-	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "Does GAMMA widget match\nthe picture below?");
-	image = gtk_image_new_from_file (filename_image);
-	vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_box_pack_end (GTK_BOX(vbox), widget, TRUE, TRUE, 12);
-	gtk_box_pack_end (GTK_BOX(vbox), image, TRUE, TRUE, 12);
-	gtk_widget_set_size_request (widget, 300, 300);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
-	gtk_widget_show (widget);
-	gtk_widget_show (image);
-
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-	/************************************************************/
-	egg_test_title (test, "plotted as expected?");
-	egg_test_assert (test, (response == GTK_RESPONSE_YES));
-
-	gtk_widget_destroy (dialog);
-
-	g_free (filename_image);
-
-	egg_test_end (test);
-}
-#endif
-
